@@ -1,5 +1,6 @@
 
 import warnings
+from typing import List, Optional, Union
 
 import paramak
 
@@ -11,87 +12,90 @@ class BallReactor(paramak.Reactor):
     most spherical reactors.
 
     Arguments:
-        inner_bore_radial_thickness (float): the radial thickness of the
-            inner bore (cm)
-        inboard_tf_leg_radial_thickness (float): the radial thickness of the
-            inner leg of the toroidal field coils (cm)
-        center_column_shield_radial_thickness (float): the radial thickness of
-            the center column shield (cm)
-        divertor_radial_thickness (float): the radial thickness of the divertor
+        inner_bore_radial_thickness: the radial thickness of the inner bore
+            (cm)
+        inboard_tf_leg_radial_thickness: the radial thickness of the inner leg
+            of the toroidal field coils (cm)
+        center_column_shield_radial_thickness: the radial thickness of the
+            center column shield (cm)
+        divertor_radial_thickness: the radial thickness of the divertor
             (cm), this fills the gap between the center column shield and
             blanket
-        inner_plasma_gap_radial_thickness (float): the radial thickness of the
+        inner_plasma_gap_radial_thickness: the radial thickness of the
             inboard gap between the plasma and the center column shield (cm)
-        plasma_radial_thickness (float): the radial thickness of the plasma
-        outer_plasma_gap_radial_thickness (float): the radial thickness of the
+        plasma_radial_thickness: the radial thickness of the plasma
+        outer_plasma_gap_radial_thickness: the radial thickness of the
             outboard gap between the plasma and firstwall (cm)
-        firstwall_radial_thickness (float): the radial thickness of the first
-            wall (cm)
-        blanket_radial_thickness (float): the radial thickness of the blanket
-            (cm)
-        blanket_rear_wall_radial_thickness (float): the radial thickness of the
-            rear wall of the blanket (cm)
-        elongation (float): the elongation of the plasma
-        triangularity (float): the triangularity of the plasma
-        plasma_gap_vertical_thickness (float): the vertical thickness of the
-            gap between the plasma and firstwall (cm). If left as None then the
-            outer_plasma_gap_radial_thickness is used. Defaults to None.
-        number_of_tf_coils (int, optional): the number of tf coils
-        pf_coil_to_rear_blanket_radial_gap (float, optional): the radial
-            distance between the rear blanket and the closest poloidal field
-            coil. Defaults to None.
-        pf_coil_radial_thicknesses (list of floats, optional): the radial
-            thickness of each poloidal field coil. Defaults to None.
-        pf_coil_vertical_thicknesses (list of floats, optional): the vertical
-            thickness of each poloidal field coil. Defaults to None.
-        pf_coil_to_tf_coil_radial_gap (float, optional): the radial distance
+        firstwall_radial_thickness: the radial thickness of the first wall (cm)
+        blanket_radial_thickness: the radial thickness of the blanket (cm)
+        blanket_rear_wall_radial_thickness: the radial thickness of the rear
+            wall of the blanket (cm)
+        elongation: the elongation of the plasma
+        triangularity: the triangularity of the plasma
+        plasma_gap_vertical_thickness: the vertical thickness of the gap
+            between the plasma and firstwall (cm). If left as None then the
+            outer_plasma_gap_radial_thickness is used.
+        divertor_to_tf_gap_vertical_thickness: the vertical thickness of the
+            gap between the divertor and the TF coils.
+        number_of_tf_coils: the number of tf coils
+        pf_coil_radial_thicknesses: the radial
+            thickness of each poloidal field coil.
+        pf_coil_vertical_thicknesses: the vertical
+            thickness of each poloidal field coil.
+        pf_coil_to_tf_coil_radial_gap: the radial distance
             between the rear of the poloidal field coil and the toroidal field
-            coil. Defaults to None.
-        pf_coil_case_thickness (float or list of floats, optional): the
-            thickness(s) to use in both the radial and vertical direction for
-            the casing around the pf coils. If float then the single value will
-            be applied to all pf coils. If list then each value will be applied
-            to the pf coils one by one. To have no casing set to 0. Defaults to
-            10.
-        outboard_tf_coil_radial_thickness (float, optional): the radial
-            thickness of the toroidal field coil. Defaults to None.
-        outboard_tf_coil_poloidal_thickness (float, optional): the poloidal
-            thickness of the toroidal field coil. Defaults to None.
-        divertor_position (str, optional): the position of the divertor,
-            "upper", "lower" or "both". Defaults to "both".
-        rotation_angle (float): the angle of the sector that is desired.
-            Defaults to 360.0.
+            coil.
+        pf_coil_radial_position: The radial (x) position(s) of the centers of
+            the poloidal field coils.
+        pf_coil_vertical_position: The vertical (z) position(s) of the centers
+            of the poloidal field coils.
+        pf_coil_case_thicknesses: the thickness(s) to use in both the radial
+            and vertical direction for the casing around the pf coils. If float
+            then the single value will be applied to all pf coils. If list then
+            each value will be applied to the pf coils one by one. To have no
+            casing set to 0.
+        outboard_tf_coil_radial_thickness: the radial thickness of the toroidal
+            field coil.
+        outboard_tf_coil_poloidal_thickness: the poloidal thickness of the
+            toroidal field coil.
+        divertor_position: the position of the divertor, "upper", "lower" or
+            "both".
+        rotation_angle: the angle of the sector that is desired.
     """
 
     def __init__(
             self,
-            inner_bore_radial_thickness,
-            inboard_tf_leg_radial_thickness,
-            center_column_shield_radial_thickness,
-            divertor_radial_thickness,
-            inner_plasma_gap_radial_thickness,
-            plasma_radial_thickness,
-            outer_plasma_gap_radial_thickness,
-            firstwall_radial_thickness,
-            blanket_radial_thickness,
-            blanket_rear_wall_radial_thickness,
-            elongation,
-            triangularity,
-            plasma_gap_vertical_thickness=None,
-            number_of_tf_coils=12,
-            pf_coil_to_rear_blanket_radial_gap=None,
-            pf_coil_radial_thicknesses=None,
-            pf_coil_vertical_thicknesses=None,
-            pf_coil_to_tf_coil_radial_gap=None,
-            pf_coil_case_thickness=10,
-            outboard_tf_coil_radial_thickness=None,
-            outboard_tf_coil_poloidal_thickness=None,
-            divertor_position="both",
-            rotation_angle=360.0,
+            inner_bore_radial_thickness: float,
+            inboard_tf_leg_radial_thickness: float,
+            center_column_shield_radial_thickness: float,
+            divertor_radial_thickness: float,
+            inner_plasma_gap_radial_thickness: float,
+            plasma_radial_thickness: float,
+            outer_plasma_gap_radial_thickness: float,
+            firstwall_radial_thickness: float,
+            blanket_radial_thickness: float,
+            blanket_rear_wall_radial_thickness: float,
+            elongation: float,
+            triangularity: float,
+            plasma_gap_vertical_thickness: Optional[float] = None,
+            divertor_to_tf_gap_vertical_thickness: Optional[float] = 0,
+            number_of_tf_coils: Optional[int] = 12,
+            rear_blanket_to_tf_gap: Optional[float] = None,
+            pf_coil_radial_thicknesses: Optional[Union[float, List[float]]] = None,
+            pf_coil_vertical_thicknesses: Optional[Union[float, List[float]]] = None,
+            pf_coil_radial_position: Optional[Union[float, List[float]]] = None,
+            pf_coil_vertical_position: Optional[Union[float, List[float]]] = None,
+            pf_coil_case_thicknesses: Optional[Union[float, List[float]]] = 10,
+            outboard_tf_coil_radial_thickness: float = None,
+            outboard_tf_coil_poloidal_thickness: float = None,
+            divertor_position: Optional[str] = "both",
+            rotation_angle: Optional[str] = 360.0,
     ):
 
         super().__init__([])
 
+        self.method = 'trelis'
+
         self.inner_bore_radial_thickness = inner_bore_radial_thickness
         self.inboard_tf_leg_radial_thickness = inboard_tf_leg_radial_thickness
         self.center_column_shield_radial_thickness = \
@@ -106,19 +110,23 @@ class BallReactor(paramak.Reactor):
         self.blanket_radial_thickness = blanket_radial_thickness
         self.blanket_rear_wall_radial_thickness = \
             blanket_rear_wall_radial_thickness
-        self.pf_coil_to_rear_blanket_radial_gap = \
-            pf_coil_to_rear_blanket_radial_gap
+
+        self.pf_coil_vertical_position = pf_coil_vertical_position
+        self.pf_coil_radial_position = pf_coil_radial_position
         self.pf_coil_radial_thicknesses = pf_coil_radial_thicknesses
         self.pf_coil_vertical_thicknesses = pf_coil_vertical_thicknesses
-        self.pf_coil_to_tf_coil_radial_gap = pf_coil_to_tf_coil_radial_gap
-        self.pf_coil_case_thickness = pf_coil_case_thickness
+        self.rear_blanket_to_tf_gap = rear_blanket_to_tf_gap
+
+        self.pf_coil_case_thicknesses = pf_coil_case_thicknesses
         self.outboard_tf_coil_radial_thickness = \
             outboard_tf_coil_radial_thickness
         self.outboard_tf_coil_poloidal_thickness = \
             outboard_tf_coil_poloidal_thickness
         self.divertor_position = divertor_position
+        self.rotation_angle = rotation_angle
 
         self.plasma_gap_vertical_thickness = plasma_gap_vertical_thickness
+        self.divertor_to_tf_gap_vertical_thickness = divertor_to_tf_gap_vertical_thickness
         if self.plasma_gap_vertical_thickness is None:
             self.plasma_gap_vertical_thickness = \
                 self.outer_plasma_gap_radial_thickness
@@ -152,6 +160,42 @@ class BallReactor(paramak.Reactor):
             self.plasma_gap_vertical_thickness,
             self.major_radius - self.minor_radius]
 
+    @property
+    def rotation_angle(self):
+        return self._rotation_angle
+
+    @rotation_angle.setter
+    def rotation_angle(self, value):
+        if value == 360:
+            msg = (
+                "360 degree rotation may result in a "
+                "Standard_ConstructionError or AttributeError"
+            )
+            warnings.warn(msg, UserWarning)
+        elif value > 360:
+            raise ValueError('rotation_angle can not be larger than 360')
+        self._rotation_angle = value
+
+    @property
+    def pf_coil_vertical_position(self):
+        return self._pf_coil_vertical_position
+
+    @pf_coil_vertical_position.setter
+    def pf_coil_vertical_position(self, value):
+        if not isinstance(value, list) and value is not None:
+            raise ValueError("pf_coil_vertical_position must be a list")
+        self._pf_coil_vertical_position = value
+
+    @property
+    def pf_coil_radial_position(self):
+        return self._pf_coil_radial_position
+
+    @pf_coil_radial_position.setter
+    def pf_coil_radial_position(self, value):
+        if not isinstance(value, list) and value is not None:
+            raise ValueError("pf_coil_radial_position must be a list")
+        self._pf_coil_radial_position = value
+
     @property
     def pf_coil_radial_thicknesses(self):
         return self._pf_coil_radial_thicknesses
@@ -189,28 +233,28 @@ class BallReactor(paramak.Reactor):
         """Creates a list of paramak.Shape for components and saves it in
         self.shapes_and_components
         """
-        shapes_and_components = []
+        uncut_shapes = []
 
-        self._rotation_angle_check()
-        shapes_and_components.append(self._make_plasma())
+        uncut_shapes.append(self._make_plasma())
         self._make_radial_build()
         self._make_vertical_build()
-        shapes_and_components.append(self._make_inboard_tf_coils())
-        shapes_and_components.append(self._make_center_column_shield())
-        shapes_and_components += self._make_blankets_layers()
-        shapes_and_components.append(self._make_divertor())
-        shapes_and_components += self._make_pf_coils()
-        shapes_and_components += self._make_tf_coils()
+        uncut_shapes.append(self._make_inboard_tf_coils())
+        uncut_shapes.append(self._make_center_column_shield())
+        uncut_shapes += self._make_blankets_layers()
+        uncut_shapes.append(self._make_divertor())
+        uncut_shapes += self._make_tf_coils()
+        pf_coils = self._make_pf_coils()
+
+        if pf_coils is None:
+            shapes_and_components = uncut_shapes
+        else:
+            for shape in uncut_shapes:
+                for pf_coil in pf_coils:
+                    shape.solid = shape.solid.cut(pf_coil.solid)
+            shapes_and_components = pf_coils + uncut_shapes
 
         self.shapes_and_components = shapes_and_components
 
-    def _rotation_angle_check(self):
-
-        if self.rotation_angle == 360:
-            msg = "360 degree rotation may result " + \
-                "in a Standard_ConstructionError or AttributeError"
-            warnings.warn(msg, UserWarning)
-
     def _make_plasma(self):
 
         plasma = paramak.Plasma(
@@ -275,85 +319,42 @@ class BallReactor(paramak.Reactor):
         # in a similar manner to the radial build
 
         self._firstwall_start_height = (
-            self._plasma.high_point[1] + self.plasma_gap_vertical_thickness
-        )
+            self._plasma.high_point[1] +
+            self.plasma_gap_vertical_thickness)
         self._firstwall_end_height = self._firstwall_start_height + \
             self.firstwall_radial_thickness
 
         self._blanket_start_height = self._firstwall_end_height
-        self._blanket_end_height = \
-            self._blanket_start_height + self.blanket_radial_thickness
+        self._blanket_end_height = self._blanket_start_height + self.blanket_radial_thickness
 
         self._blanket_rear_wall_start_height = self._blanket_end_height
-        self._blanket_rear_wall_end_height = (
-            self._blanket_rear_wall_start_height +
-            self.blanket_rear_wall_radial_thickness)
+        self._blanket_rear_wall_end_height = self._blanket_rear_wall_start_height + \
+            self.blanket_rear_wall_radial_thickness
 
-        self._tf_coil_height = self._blanket_rear_wall_end_height
-        self._center_column_shield_height = \
-            self._blanket_rear_wall_end_height * 2
+        self._tf_coil_start_height = self._blanket_rear_wall_end_height + \
+            self.divertor_to_tf_gap_vertical_thickness
 
-        if (self.pf_coil_vertical_thicknesses,
-                self.pf_coil_radial_thicknesses,
-                self.pf_coil_to_rear_blanket_radial_gap) != (None, None, None):
-            self._number_of_pf_coils = len(self.pf_coil_vertical_thicknesses)
+        self._center_column_shield_height = self._blanket_rear_wall_end_height * 2
 
-            y_position_step = (2 * (
-                self._blanket_rear_wall_end_height
-                + self.pf_coil_to_rear_blanket_radial_gap
-            )
-            ) / (self._number_of_pf_coils + 1)
-
-            if not isinstance(self.pf_coil_case_thickness, list):
-                self.pf_coil_case_thickness = [
-                    self.pf_coil_case_thickness] * self._number_of_pf_coils
-
-            self._pf_coils_xy_values = []
-            # adds in coils with equal spacing strategy, should be updated to
-            # allow user positions
-            for i in range(self._number_of_pf_coils):
-                y_value = (
-                    self._blanket_rear_wall_end_height
-                    + self.pf_coil_to_rear_blanket_radial_gap
-                    - y_position_step * (i + 1)
-                )
-                x_value = (
-                    self._blanket_rear_wall_end_radius
-                    + self.pf_coil_to_rear_blanket_radial_gap
-                    + 0.5 * self.pf_coil_radial_thicknesses[i]
-                    + self.pf_coil_case_thickness[i]
-                )
-                self._pf_coils_xy_values.append((x_value, y_value))
-
-            self._pf_coil_start_radius = (
-                self._blanket_rear_wall_end_radius +
-                self.pf_coil_to_rear_blanket_radial_gap)
-
-            self._pf_coil_end_radius = self._pf_coil_start_radius + \
-                max(self.pf_coil_radial_thicknesses) + \
-                max(self.pf_coil_case_thickness) * 2
-
-            if (self.pf_coil_to_tf_coil_radial_gap,
-                    self.outboard_tf_coil_radial_thickness) != (None, None):
-                self._tf_coil_start_radius = (
-                    self._pf_coil_end_radius +
-                    self.pf_coil_to_rear_blanket_radial_gap)
-                self._tf_coil_end_radius = (
-                    self._tf_coil_start_radius +
-                    self.outboard_tf_coil_radial_thickness)
+        if self.rear_blanket_to_tf_gap is not None:
+            self._tf_coil_start_radius = self._blanket_rear_wall_end_radius + \
+                self.rear_blanket_to_tf_gap
+            self._tf_coil_end_radius = (
+                self._tf_coil_start_radius +
+                self.outboard_tf_coil_radial_thickness)
 
     def _make_inboard_tf_coils(self):
 
         self._inboard_tf_coils = paramak.CenterColumnShieldCylinder(
-            height=self._tf_coil_height * 2,
+            height=self._tf_coil_start_height * 2,
             inner_radius=self._inboard_tf_coils_start_radius,
             outer_radius=self._inboard_tf_coils_end_radius,
             rotation_angle=self.rotation_angle,
-            # color=centre_column_color,
             stp_filename="inboard_tf_coils.stp",
             stl_filename="inboard_tf_coils.stl",
             name="inboard_tf_coils",
             material_tag="inboard_tf_coils_mat",
+            color=(0, 0, 1)
         )
         return self._inboard_tf_coils
 
@@ -364,7 +365,7 @@ class BallReactor(paramak.Reactor):
             inner_radius=self._center_column_shield_start_radius,
             outer_radius=self._center_column_shield_end_radius,
             rotation_angle=self.rotation_angle,
-            # color=centre_column_color,
+            color=(0., 0.333, 0.),
             stp_filename="center_column_shield.stp",
             stl_filename="center_column_shield.stl",
             name="center_column_shield",
@@ -379,7 +380,8 @@ class BallReactor(paramak.Reactor):
             height=self._center_column_shield_height * 1.5,
             inner_radius=0,
             outer_radius=self._center_column_shield_end_radius,
-            rotation_angle=360
+            rotation_angle=360,
+            color=(0., 0., 1.)
         )
 
         self._firstwall = paramak.BlanketFP(
@@ -392,6 +394,8 @@ class BallReactor(paramak.Reactor):
             material_tag="firstwall_mat",
             stp_filename="firstwall.stp",
             stl_filename="firstwall.stl",
+            color=(0.5, 0.5, 0.5),
+            name='firstwall',
             cut=[self._center_column_cutter]
         )
 
@@ -406,6 +410,8 @@ class BallReactor(paramak.Reactor):
             material_tag="blanket_mat",
             stp_filename="blanket.stp",
             stl_filename="blanket.stl",
+            color=(0., 1., 0.498),
+            name='blanket',
             cut=[self._center_column_cutter])
 
         self._blanket_rear_wall = paramak.BlanketFP(
@@ -420,6 +426,8 @@ class BallReactor(paramak.Reactor):
             material_tag="blanket_rear_wall_mat",
             stp_filename="blanket_rear_wall.stp",
             stl_filename="blanket_rear_wall.stl",
+            color=(0., 1., 1.),
+            name='blanket_rear_wall',
             cut=[self._center_column_cutter],
         )
 
@@ -459,6 +467,7 @@ class BallReactor(paramak.Reactor):
             stl_filename="divertor.stl",
             name="divertor",
             material_tag="divertor_mat",
+            color=(1., 0.667, 0.),
             rotation_angle=self.rotation_angle
         )
 
@@ -471,45 +480,64 @@ class BallReactor(paramak.Reactor):
         return self._divertor
 
     def _make_pf_coils(self):
-        list_of_components = []
-        if (self.pf_coil_vertical_thicknesses,
-                self.pf_coil_radial_thicknesses,
-                self.pf_coil_to_rear_blanket_radial_gap) != (None, None, None):
+        if None not in [self.pf_coil_vertical_thicknesses,
+                        self.pf_coil_radial_thicknesses,
+                        self.pf_coil_vertical_position,
+                        self.pf_coil_radial_position]:
+            list_of_components = []
 
-            self._pf_coil = paramak.PoloidalFieldCoilSet(
+            # TODO make use of counter in the name attribute
+
+            center_points = [
+                (x, y) for x, y in zip(
+                    self.pf_coil_radial_position, self.pf_coil_vertical_position)]
+
+            self._pf_coils = self._pf_coil = paramak.PoloidalFieldCoilSet(
                 heights=self.pf_coil_vertical_thicknesses,
                 widths=self.pf_coil_radial_thicknesses,
-                center_points=self._pf_coils_xy_values,
+                center_points=center_points,
                 rotation_angle=self.rotation_angle,
                 stp_filename='pf_coils.stp',
                 stl_filename='pf_coils.stl',
                 name="pf_coil",
                 material_tag="pf_coil_mat",
             )
+            list_of_components.append(self._pf_coil)
 
-            self._pf_coils_casing = paramak.PoloidalFieldCoilCaseSetFC(
-                pf_coils=self._pf_coil,
-                casing_thicknesses=self.pf_coil_case_thickness,
-                rotation_angle=self.rotation_angle,
-                stp_filename='pf_coil_cases.stp',
-                stl_filename='pf_coil_cases.stl',
-                name="pf_coil_case",
-                material_tag="pf_coil_case_mat",
-            )
+            if self.pf_coil_case_thicknesses is not None:
+                self._pf_coils_casing = paramak.PoloidalFieldCoilCaseSetFC(
+                    pf_coils=self._pf_coil,
+                    casing_thicknesses=self.pf_coil_case_thicknesses,
+                    rotation_angle=self.rotation_angle,
+                    stp_filename='pf_coil_cases.stp',
+                    stl_filename='pf_coil_cases.stl',
+                    name="pf_coil_case",
+                    material_tag="pf_coil_case_mat",
+                )
+                list_of_components.append(self._pf_coils_casing)
 
-            list_of_components = [self._pf_coils_casing, self._pf_coil]
-        return list_of_components
+            return list_of_components
+        else:
+            print(
+                'pf_coil_vertical_thicknesses, pf_coil_radial_thicknesses, '
+                'pf_coil_radial_position, pf_coil_vertical_position not '
+                'so not making pf coils')
+            return None
 
     def _make_tf_coils(self):
         comp = []
-        if (self.pf_coil_to_tf_coil_radial_gap,
-                self.outboard_tf_coil_radial_thickness) != (None, None):
+        # checks that all the required information has been input by the user
+        if None not in [
+                self.rear_blanket_to_tf_gap,
+                self.outboard_tf_coil_radial_thickness,
+                self.outboard_tf_coil_poloidal_thickness,
+                self.number_of_tf_coils] and self.number_of_tf_coils > 1:
 
             self._tf_coil = paramak.ToroidalFieldCoilRectangle(
                 with_inner_leg=False,
                 horizontal_start_point=(
                     self._inboard_tf_coils_start_radius,
-                    self._tf_coil_height),
+                    self._tf_coil_start_height),
                 vertical_mid_point=(
                     self._tf_coil_start_radius, 0),
                 thickness=self.outboard_tf_coil_radial_thickness,
