@@ -1,5 +1,5 @@
 
-from typing import Optional
+from typing import Optional, Tuple
 
 from paramak import Shape
 
@@ -20,12 +20,14 @@ class RotateMixedShape(Shape):
         rotation_angle: Optional[float] = 360.0,
         stp_filename: Optional[str] = "RotateMixedShape.stp",
         stl_filename: Optional[str] = "RotateMixedShape.stl",
+        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.121, 0.47, 0.705),
         **kwargs
     ):
 
         super().__init__(
             stp_filename=stp_filename,
             stl_filename=stl_filename,
+            color=color,
             **kwargs
         )
         self.rotation_angle = rotation_angle
